// Copyright 1999-2017. Parallels IP Holdings GmbH. All Rights Reserved.

(function (cssPrefix) {

    PleskExt.Git.CommitLog = Class.create(Jsw.List, {
        _initConfiguration: function ($super, config) {
            config = Object.extend({
                cls: cssPrefix + 'commit-log'
            }, config || {});
            $super(config);

            this._initColumns();
            this._initOperations();
        },

        _initColumns: function() {
            this._columns.push({
                header: this.lmsg('date'),
                dataIndex: 'date',
                cls: 'min nowrap'
            });
            this._columns.push({
                header: this.lmsg('hash'),
                dataIndex: 'hash',
                cls: 'min'
            });
            this._columns.push({
                header: this.lmsg('userNameEmail').escapeHTML(),
                dataIndex: 'userNameEmail',
                renderer: function (item) {
                    return item.userName.escapeHTML() + ' ' +
                        new Element('a', {
                            href:'mailto:' + encodeURIComponent(item.userName + ' ' + '<' + item.userEmail + '>')}
                        ).update(('<' + item.userEmail + '>').escapeHTML()).outerHTML;
                }
            });
            this._columns.push({
                header: this.lmsg('message'),
                dataIndex: 'message',
                cls: 'message',
                renderer: function (item) {
                    var text = item.message.trim().escapeHTML();

                    var rows = text.split("\n");
                    var visibleNumber = 1;
                    if (rows.length <= visibleNumber) {
                        return text.replace(/\n/g, "<br/>");
                    }
                    var firstRows = rows.slice(0, visibleNumber);
                    var lastRows = rows.slice(visibleNumber);
                    var more = '<a class="toggler js-more-link" href="#"><span>' + this.lmsg('more') + '&gt;&gt;</span></a>';
                    var less = '<a class="toggler js-more-link" href="#" style="display: none;"><span>&lt;&lt;' + this.lmsg('less') + '</span></a>';
                    return firstRows.join('<br/>') + '<div class="js-more" style="display: none;">' + lastRows.join('<br/>') + '</div>' +
                        '<div>' + more + less + '</div>';
                }.bind(this)
            });

        },

        _initOperations: function() {
            this._operations = [
                {
                    componentType: 'Jsw.SmallButton',
                    title: this.lmsg('refreshButton'),
                    description: this.lmsg('refreshButtonHint'),
                    addCls: cssPrefix + 'sb-refresh',
                    handler: function() {
                        this.reload();
                    }.bind(this)
                }
            ];
        },

        _addEvents: function($super) {
            $super();
            this._componentElement.observe('click', function(event) {
                var element = event.findElement('a.js-more-link');
                if (!element) {
                    return;
                }
                event.preventDefault();
                element.up('td').down('.js-more').toggle();
                element.up('div').select('a').invoke('toggle');
            });
        }
    });

})(PleskExt.Git.PleskExtPrefix);
