// Copyright 1999-2017. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('PleskExt.Git');

(function (ce, render, skinUrl) {
    PleskExt.Git.InPlaceEditor = Class.create({
        _titleText: null,
        _descriptionText: null,
        _okText: null,
        _cancelText: null,
        initialize: function (config) {
            this._config = config;
            this._titleText = this._getConfigParam('titleText');
            this._descriptionText = this._getConfigParam('descriptionText');
            this._okText = this._getConfigParam('okText');
            this._cancelText = this._getConfigParam('cancelText');
            this._okCallback = this._getConfigParam('okCallback');
        },
        show: function (targetEl) {
            if (!this._componentElement) {
                this._render();
            }
            this._componentElement.removeClassName('collapsed');
            if (targetEl) {
                this._updatePosition(targetEl);
            }
        },
        hide: function () {
            if (!this._componentElement) {
                return;
            }
            this._componentElement.addClassName('collapsed');
        },
        visible: function () {
            return this._componentElement && !this._componentElement.hasClassName('collapsed');
        },
        _updatePosition: function (targetEl) {
            this._componentElement.clonePosition(targetEl, {
                offsetLeft: - 50 + targetEl.getWidth() / 2,
                offsetTop: - this._componentElement.getHeight(),
                setWidth: false,
                setHeight: false
            });
        },
        _renderContent: function () {
            return '';
        },
        _render: function () {
            var id = this._getRandomId();
            render(document.body,
                ce('.popup-box.top-right.collapsed#' + id,
                    this._renderHead(),
                    ce(".popup-content",
                        ce(".popup-content-area", [,
                            ce("span.arrow"),
                            this._renderContentWrapper(this._renderContent()),
                            this._renderButtons()
                        ])
                    )
                )
            );
            this._componentElement = $(id);
            this._componentElement.observe('click', function (event) {
                event.stopPropagation();
            });
        },
        _renderHead: function () {
            return ce(".popup-heading", [ce("span.close", {
                'onclick': function (event) {
                    this.hide();
                }.bind(this)
            }), ce("h4", ce("span", this._titleText))]);
        },
        _renderButtons: function () {
            return ce(".btns-box",
                ce(".box-area",
                    ce(".form-row",
                        ce(".field-value", [
                            ce("span.btn.action", ce("button[type=button]", {'onclick': this.onClickOk.bind(this)}, this._okText)),
                            ce("span.btn", ce("button[type=button]", {'onclick': this.onClickCancel.bind(this)}, this._cancelText))
                        ])
                    )
                )
            );
        },
        _renderContentWrapper: function (content) {
            return ce(".form-box",
                ce(".content",
                    ce(".content-area",
                        ce(".form-row", [
                            ce(".field-name", ce("label", this._descriptionText)),
                            ce(".field-value", ce(".text-value", content))
                        ])
                    )
                )
            );
        },
        onClickOk: function (event) {
            event.preventDefault();
            this.hide();
            var v = this.getCurrentValue();
            this._okCallback(v);
        },
        onClickCancel: function (event) {
            event.preventDefault();
            this.hide();
        },
        getCurrentValue: function () {
            return null;
        },
        _getRandomId: function () {
            var id = 'gen-id-' + this._getRandomNumber();

            while (null !== $(id)) {
                id = 'gen-id-' + this._getRandomNumber();
            }

            return id;
        },
        _getRandomNumber: function () {
            return Math.floor(1000000 * Math.random());
        },
        _hasConfigParam: function (name) {
            return 'undefined' !== typeof this._config[name];
        },
        _getConfigParam: function (name, defaultValue) {
            return this._hasConfigParam(name)
                ? this._config[name]
                : defaultValue;
        }
    });

    PleskExt.Git.DeploymentMode = Class.create(PleskExt.Git.InPlaceEditor, {
        _modes: null,
        _defaultMode: null,
        initialize: function ($super, config) {
            $super(config);
            this._modes = this._getConfigParam('modes');
            this._defaultMode = this._getConfigParam('defaultMode');
        },
        _renderContent: function () {
            return Object.keys(this._modes).map(function (modeKey) {
                var mode = this._modes[modeKey];
                var props = {'type': "radio", 'name': "deploymendModeRadio", 'value': mode.value, 'id': mode.value};
                if (this._defaultMode === mode.value) {
                    props['checked'] = true;
                }
                return ce(".indent-box", [
                    ce("input.radio", props, ''),
                    ce(".indent-box-content", [
                        ce("label", {'for': mode.value}, mode.title),
                        ce("p.hint", mode.hint)
                    ])
                ]);
            }.bind(this));
        },
        getCurrentValue: function () {
            return this._componentElement.down('input:checked[type=radio][name=deploymendModeRadio]').value;
        }
    });

    PleskExt.Git.DeploymentPath = Class.create(PleskExt.Git.InPlaceEditor, {
        _tree: null,
        _rootDirTitle: null,
        _path: null,
        _createFolderText: null,
        initialize: function ($super, config) {
            $super(config);
            this._rootDirTitle = this._getConfigParam('rootDirTitle');
            this._path = this._getConfigParam('path');
            this._createFolderText = this._getConfigParam('createFolderText');
        },
        _renderContent: function () {
            return ce(".text-value",
                ce('div.folder-selector',
                    ce('div.scrollable fm-scrollable', ce('div.scrollable-wrap', ce('div.tree#git-files-tree'))),
                    ce('a.btn', {href: '#', onclick: this._onCreateClick.bind(this)}, this._createFolderText)
                )
            );
        },
        _render: function ($super) {
            $super();
            this._tree = new Jsw.FileManager.Tree({
                applyTo: 'git-files-tree',
                data: null,
                dataUrl: '/smb/file-manager/tree-data',
                createFolderUrl: '/file-manager/create-directory',
                rootNodeTitle: this._rootDirTitle,
                'autoRender': false,
                onReload: function () {
                    var pathParts = this._path.split('/');
                    this._tree.setDirectory('/' + pathParts[0]);
                }.bind(this)
            });
        },
        getCurrentValue: function () {
            return this._tree.getSelectedItemData().path;
        },
        _onCreateClick: function(event) {
            event.preventDefault();
            this._tree.hideNewNode();
            this._tree.showNewNode();
            var selectedNode = this._tree.getSelectedNode();
            if (selectedNode) {
                $('git-files-tree').up('.fm-scrollable').scrollTop = selectedNode.offsetTop;
            }
        }
    });

    PleskExt.Git.DeploymentSettings = Class.create({
        data: null,
        parentElement: null,
        currentMode: null,
        path: null,
        deploymentMode: null,
        initialize: function (parentElement, data) {
            this.data = data;
            this.parentElement = parentElement;
            this.currentMode = this.data.defaultMode;
            this.path = this.data.defaultPath;

            this.inPlaceEditors = {
                deploymentMode: new PleskExt.Git.DeploymentMode({
                    'modes': this.data.modes,
                    'defaultMode': this.data.defaultMode,
                    'titleText': this.data.deploymentModeTitle,
                    'descriptionText': this.data.deploymentModeDescription,
                    'okText': this.data.popUpOk,
                    'cancelText': this.data.popUpCancel,
                    'okCallback': this.deploymentModeCallback.bind(this)
                })};
            if (this.data.isPathPopup) {
                this.inPlaceEditors.deploymentPath = new PleskExt.Git.DeploymentPath({
                    'rootDirTitle': this.data.rootDirectoryTitle,
                    'path': this.data.defaultPath,
                    'titleText': this.data.deploymentPathTitle,
                    'descriptionText': this.data.deploymentPathDescription,
                    'okText': this.data.popUpOk,
                    'cancelText': this.data.popUpCancel,
                    'okCallback': this.deploymentPathCallback.bind(this),
                    'createFolderText': this.data.deploymentPathCreateFolder
                });
            }

            $(document.body).observe('click', this.hideInPlaceEditors.bind(this));
        },
        render: function () {
            var mode = this.data.modes[this.currentMode];
            var elements = [
                ce("input", {"name": this.data.namePrefix + "[deploymentMode]", "type": "hidden", "value": this.currentMode}),
                (this.data.isPathPopup ? ce("input", {"name": this.data.namePrefix + "[deploymentPath]", "type": "hidden", "value": this.path}) : ''),
                mode.head,
                ce('br'),
                ce("a.toggler", {'onclick': this.toggleInPlaceEditor.bind(this, 'deploymentMode')}, mode.linkText)
            ];
            if (mode.showPath) {
                elements = elements.concat(ce('div.form-row', ce('div.single-row', ce('div#git-deploymentPath',
                    this.data.isPathPopup ? [
                        this.data.to,
                        ce('br'),
                        ce(
                            "a.toggler",
                            {'onclick': this.toggleInPlaceEditor.bind(this, 'deploymentPath')},
                            '/' + Jsw.escapeAttribute(this.path.replace(/^\/|\/$/g, '')) + '/'
                        ),
                        this.data.directory
                    ] : [
                        this.data.toLong,
                        ce('br'),
                        ce("span", [
                            ce("img", {'src': skinUrl + 'icons/16/plesk/website.png'}),
                            ' / ',
                            ce("input.f-middle-size.input-text", {
                                'name':this.data.namePrefix + '[deploymentPath]',
                                'value': this.path.replace(/^\/|\/$/g, '') + '/'
                            })
                        ])
                    ],
                    ce('span.field-errors', {style: 'display: none;'})
                ))));
            }

            render(this.parentElement, elements, 'inner');
        },
        toggleInPlaceEditor: function (name, event) {
            event.stopPropagation();
            var editor = this.inPlaceEditors[name];
            var visible = editor.visible();
            this.hideInPlaceEditors();
            if (!visible) {
                editor.show(event.target);
            }
        },
        hideInPlaceEditors: function () {
            Object.keys(this.inPlaceEditors).forEach(function (name) {
                var editor = this.inPlaceEditors[name];
                if (editor.visible()) {
                    editor.hide();
                }
            }.bind(this));
        },
        deploymentModeCallback: function (value) {
            this.currentMode = value;
            this.render();
        },
        deploymentPathCallback: function (value) {
            this.path = value;
            this.render();
        }
    });
})(Jsw.createElement, Jsw.render, Jsw.skinUrl);
